#ifndef _TIME_H
#include <time.h>
#endif
#include <glib.h>

struct config_t {
	GString	*datapath;
	int		datamode;
	int		dirmode;
};
extern struct config_t g_Config;

#define MAKROKOSMOS_DATA_PATH "/var/lib/makrokosmos/"
#define MAKROKOSMOS_DATA_MODE 0660
#define MAKROKOSMOS_DIR_MODE 02770
#define null_config_t (struct config_t) { \
	datapath:	g_string_new(MAKROKOSMOS_DATA_PATH),	\
	datamode:	MAKROKOSMOS_DATA_MODE,	\
	dirmode:	MAKROKOSMOS_DIR_MODE	\
	}
#define MAKROKOSMOS_PLAYER_PATH "players/"
#define MAKROKOSMOS_SECTOR_PATH "sectors/"
#define MAKROKOSMOS_FLEET_PATH "fleets/"
#define MAKROKOSMOS_SHIP_PATH "ships/"
#define MAKROKOSMOS_ITEM_PATH "items/"
#define MAKROKOSMOS_SPECIES_PATH "species/"
#define MAKROKOSMOS_RECIPE_PATH "recipes/"
#define MAKROKOSMOS_MAIL_PATH "mail/"
#define MAKROKOSMOS_CHARTS_PATH "charts/"
#define MAKROKOSMOS_HELP_PATH "help/"
#define MAKROKOSMOS_SETTLEMENT_TEMPLATE "Defaults/NewSettlement"
//#define MAKROKOSMOS_COLONY_TEMPLATE "Defaults/NewColony"
#define MAKROKOSMOS_OUTPOST_TEMPLATE "Defaults/NewOutpost"
#define MAKROKOSMOS_CONFIG_FILE "makrokosmos.cfg"
#define MAKROKOSMOS_LOG_PRIVATE "private.log"
#define MAKROKOSMOS_LOG_PUBLIC "public.log"
#define MAKROKOSMOS_CHAT_PATH "chat/"
#define MAKROKOSMOS_CHAT_FILE "chat.log"

/* Cycle length is in seconds. */
#define	MAKROKOSMOS_CYCLE_LENGTH	3600
#define MAKROKOSMOS_TURNS_PER_CYCLE	15
#define MAKROKOSMOS_NEWPLAYER_TURNS	MAKROKOSMOS_TURNS_PER_CYCLE*72;
#define	MAKROKOSMOS_NOW			(time(NULL)/MAKROKOSMOS_CYCLE_LENGTH)
#define	MAKROKOSMOS_MAX_AGING		16384

#define	MAKROKOSMOS_NEWPLAYER_POP	800

/* Number of Defaults/Template# files in species dir */
#define MAKROKOSMOS_SPECIES_TEMPLATES	3

#define MAKROKOSMOS_MAX_DESCLEN			10

struct body_t {
  int		class;
  GString	*name;
  GString	*owner;
  GString	*desc;
  int		numofleets;
  GString	**ofleets;
  int		numlfleets;
  GString	**lfleets;
  int		numcolonies;
  GString	**colonies;
  int		primary;
  double	mintemp;
  double	maxtemp;
  double	mass;
  double	grav;
  int		minradius;
  int		radius;
  int		maxradius;
  int		numzones;
  GString	**zones;
  float		orbradius;
  float		perigee;
  float		apogee;
  float		orbecc;
  float		orbinc;
  float		day;	/* In seconds */
  float		year;	/* In Earth-days */
  float		eqinc;	/* Equatorial inclination */
  int		retrograde;
};

#define null_body (struct body_t) { class: 0, \
  name: g_string_new(""), \
  owner: g_string_new(""), desc: g_string_new(""), \
  numofleets: 0, ofleets: NULL, numlfleets: 0, lfleets: NULL, \
  numcolonies: 0, colonies: NULL, \
  primary: -1, mintemp: 0, maxtemp: 0, mass: 0, grav: 0, \
  minradius: 0, radius: 0, maxradius: 0, numzones: 0, zones: NULL, \
  orbradius: 0, perigee: 0, apogee: 0, orbecc: 0, orbinc: 0, \
  day: 0, year: 0, eqinc: 0, retrograde: 0 \
  }

/*
 *   Body Classes
 */
#define	BCLASS_STAR		1
#define BCLASS_BROWNDWARF	2
#define BCLASS_GASGIANT		3
#define BCLASS_TERRESTRIAL	4
#define BCLASS_WHITEDWARF	5
#define BCLASS_NEUTRONSTAR	6
#define BCLASS_BLACKHOLE	7
#define BCLASS_WORMHOLE		8

struct sector_t {
  GString	*name;
  GString	*owner;
  GString	*comments;
  int		numbodies;
  struct body_t	*bodies;
  int		numfleets;
  GString	**fleets;
};

#define null_sector (struct sector_t) { name: g_string_new(""), \
	owner: g_string_new(""), comments: g_string_new(""), \
        numbodies: 0, bodies: NULL, numfleets: 0, fleets: NULL }

struct tech_t {
  int	bio;
  int	emf;		/* Electromagnetic */
  int	grav;		/* Gravitotemporal */
  int	logic;
  int	mech;
  int	psi;		/* Psionomagical */
  int	quantum;
};

#define null_tech (struct tech_t) { bio: 0, emf: 0, grav: 0, logic: 0, \
	mech: 0, psi: 0, quantum: 0 }

struct theme_t {
  int	custom;
  int	chart_blank;
};

#define null_theme (struct theme_t) { custom: 0, chart_blank: 0x808000 }

struct player_t {
  GString	*name;
  GString	*coords;
  GString	*dest;
  GString	*fleet;
  GString	*dock;
  GString	*overlords;
  GString	*accesscode;
  struct theme_t theme;
  int		orbit;
  int		locus;
  int		turns;
  int		turnsupdated;
  int		researchupdated;
  struct tech_t	tech;
  time_t	logread;
  int		biopoints;
};

#define null_player	(struct player_t){ name: g_string_new(""), \
	coords: g_string_new("0/0/0"), dest: g_string_new(""), \
	fleet: g_string_new(""), dock: g_string_new(""), \
    overlords: g_string_new(""), \
	accesscode: g_string_new(""), \
    theme: null_theme, \
	orbit: 0, locus: LOCUS_SECTOR, \
    turns: 0, turnsupdated: time(NULL)/MAKROKOSMOS_CYCLE_LENGTH, \
	researchupdated: MAKROKOSMOS_NOW, tech: null_tech, \
	logread: 0, biopoints: 0 }

#define LOCUS_SECTOR		0
#define LOCUS_ORBIT         	1
#define LOCUS_LANDED		2
#define	LOCUS_DOCKED		3
#define LOCUS_HYPERSPACE	4

struct item_t {
/*  void		*ship;	*/	/* Parent */
  GString	*base;
  struct tech_t	buildtech;
  int		buildtime;
  int		class;
  int		surface;	/* Customizable */
  int		mass;
  int		volume;		/* Customizable */
  int		mincrew;
  int		maxcrew;
  int		crew;
  int		maxhp;		/* Customizable */
  int		hp;         /* Modifiable */
  int		strength;
  int		flux;		/* Random modifier for strength */
  int		regentime;
  int		cargomax;
  int		cargoqty;	/* Modifiable */
  int		cargouse;
  GString	*cargo;		/* Customizable */
  int		updated;	/* Modifiable */
  int		numpopstrs;
  GString	**popstrs;	/* Modifiable */
  int		defaultequipped;
  int		equipped;	/* Modifiable */
  int		add_on;
  int		modified;
  int		cyclestocompletion;
  int		immobile;
};

/* Modified states are:
	0: Not modified
	1: Modified	*/
#define	MODDED_CUSTOM	2

#define null_item	(struct item_t) { /*ship: NULL, */ \
	base: g_string_new(""), \
	buildtech: null_tech, buildtime: 1, \
	class: 0, \
	surface: 0, mass: 0, volume: 0, \
	mincrew: 0, maxcrew: 0, crew: 0, maxhp: 0, hp: 0, \
	strength: 0, flux: 0, regentime: 0, \
	cargomax: 0, cargoqty: 0, cargouse: 0, cargo: g_string_new(""), \
	updated: 0, numpopstrs: 0, popstrs: NULL, \
	equipped: 0, add_on: 0, modified: 0, cyclestocompletion: 0, \
	immobile: 0 }
	/*
	 * null_item.updated used to be MAKROKOSMOS_NOW.
	 */

#define	ICLASS_DESTROYED	-1
#define ICLASS_WEAPON		1
#define	ICLASS_ARMOUR		2
#define	ICLASS_HABITAT		3
#define	ICLASS_HOLD		4
#define	ICLASS_ORBITDRIVE	7
#define	ICLASS_FLEETSENSOR	8
#define	ICLASS_FACTORY		9
#define ICLASS_DRAINER		10
#define	ICLASS_TARGETINGCOMP	11
#define	ICLASS_ANTIINERTIA	12
#define	ICLASS_PSIWEAPON	13
#define	ICLASS_SHIPYARD		14
#define	ICLASS_AIRMINER		15
#define ICLASS_BODYSENSOR	16
#define ICLASS_TELEPORT_RECALL	17
#define ICLASS_TELEPORT_RECEIVE	18
#define ICLASS_TELEPORT_SEND	19
#define	ICLASS_REPAIR_BAY	20
#define ICLASS_ENVIRONMENT	21

#define destroyed_item	(struct item_t) { \
	base: g_string_new(""), \
	buildtech: null_tech, buildtime: 0, \
	class: ICLASS_DESTROYED, \
	surface: 0, mass: 0, volume: 0, \
	mincrew: 0, maxcrew: 0, crew: 0, maxhp: 0, hp: 0, \
	strength: 0, flux: 0, regentime: 0, \
	cargomax: 0, cargoqty: 0, cargouse: 0, cargo: g_string_new(""), \
	updated: MAKROKOSMOS_NOW, numpopstrs: 0, popstrs: NULL, \
	equipped: 0, add_on: 0, modified: 1, immobile: 1 }

struct ship_t {
  void		*fleet;		/* Parent */
  GString	*base;
  int		volume;		/* Customizable */
  int		surface;	/* Customizable */
  int		maxhp;		/* Customizable */
  int		hp;		/* Modifiable */
  int		modified;	/* Modifiable */
  int		numitems;	/* Modifiable (indirect) */
  struct item_t *items;		/* Modifiable (indirect) */
  int		curritem;	/* Scratch var */
  int       	immobile;   /* Colonies, etc... */
};

#define null_ship (struct ship_t) { fleet: NULL, \
	base: g_string_new(""), \
	volume: 0, surface: 0, \
	maxhp: 0, hp: 0, \
	modified: 0, numitems: 0, items: NULL, curritem: -1, immobile: 0 }

struct fleet_t {
  int		numships;
  struct	ship_t *ships;
  GString	*coords;
  GString	*desc;
  struct sector_t *sect;  
  int		locus;
  int		body;
  int		zone;
  int		immobile;
};

#define null_fleet (struct fleet_t) { numships: 0, ships: NULL, \
	coords: g_string_new(""), desc: g_string_new(""), \
	sect: NULL, locus: LOCUS_SECTOR, body: 0, zone: 0, immobile: 0 }

struct species_t {
  GString	*name;
  GString	*genders;
  int		sexes;
  int		reproducenum;
	/*
	 * if sexes > reproducenum, higher sexes are neuter.
         * if reproducenum>sexes, is asexual and just needs reprodunum
         * fertile individuals.
         */
  double	reproduceodds;
  int		puberty;
  int		menopause;
  int		senescence;
  double	mortality;
  int		hp;
  int		mintemp;
  int		maxtemp;
  float		minpress;
  float		maxpress;
  GString	*habitat;
	/* mintemp, maxtemp, minpress, and maxpress deprecated in favour of a string. */
};

#define null_species	(struct species_t) { name: g_string_new(""), \
	genders: g_string_new(""), \
	sexes: 0, reproducenum: 0, reproduceodds: 0, \
	puberty: 0, menopause: 0, \
	senescence: 0, mortality: 0, hp: 0, \
        habitat: g_string_new("") }

struct species_params_t {
	int points;
	int temptolerance;
	int pressuretolerance;
	float basepressure;
};

#define null_species_params (struct species_params_t) {	\
	points: 0, temptolerance: 0, pressuretolerance: 0 \
	}

struct pop_t {
  struct species_t	species;
  int			total;
  int			**num;
};

#define null_pop	(struct pop_t) { \
	species: null_species, total: 0, num: NULL \
	}

struct populii_t {
  int lastupdated;
  int total;
  int numpops;
  struct pop_t *pops;
};

#define	null_populii	(struct populii_t) { lastupdated: 0, \
	total: 0, numpops: 0, pops: NULL }

extern struct player_t g_Player;

struct recipe_t {
  struct tech_t	tech;
  int		time;
  int		numingredients;
  GString	**ingredient;
  int		*ingredientqty;
};

#define	null_recipe	(struct recipe_t) { tech: null_tech, time: 1, \
	numingredients: 0, ingredient: NULL, ingredientqty: NULL \
	}

struct item_p {
  int	ship;
  int	item;
};

struct itemlist_t {
  int		listlen;
  int		alloced;
  struct item_p *list;
};

#define null_itemlist (struct itemlist_t) { listlen: 0, alloced: 0, list: NULL }

struct zone_t {
	double	mintemp;
	double	maxtemp;
	float	minpress;
	float	maxpress;
	int	numchems;
	GString	**chems;
        int	*amts;
};

#define null_zone	(struct zone_t) { \
	mintemp: 0, maxtemp: 0, minpress: 0, maxpress: 0, numchems: 0, \
	chems: NULL, amts: NULL \
	}

#define	ERR_TOOBIG	-1001
#define ERR_TOOADVANCED	-1002

extern int g_Config_Horizon;
extern int g_Config_Systems;
extern int g_Config_MaxSystems;

struct habitatreq_item {
	GString *chem;
	char	cond;
	char	pad[3];
	int		amt;
};
