/*
 * (Communications Code)
 */

#define __USE_GNU

#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <glib.h>
#include <errno.h>
#include <string.h>
#include "microcosm.h"
#include "comm.h"
#include "util.h"
#include "ui_all.h"

void send_mail(char *sender, char *recipient, char *mail)
{
  GString	*spool=g_string_new(g_Config.datapath->str);
  int		fd;
  FILE		*f;
  g_string_append(spool,MAKROKOSMOS_MAIL_PATH);
  g_string_append(spool,recipient);
  fd=open(spool->str,O_WRONLY|O_APPEND|O_CREAT, g_Config.datamode);
  if (fd>=0) {
    if (flock(fd,LOCK_EX)==0) {
      fchmod(fd, g_Config.datamode);
      lseek(fd,0,SEEK_END);
      f=fdopen(fd,"a");
      if (f) {
        fprintf(f,"From %s\nTo: %s\n\n%s\n.\n",sender,recipient,mail);
        fclose(f);
      } else {
        ui_config_error("Error opening mail spool as stream.",errno);
      }
      flock(fd,LOCK_UN);
    } else {
      ui_message("Error locking mail spool.");
    }
    close(fd);
  } else {
    ui_config_error("Error opening mail spool.",errno);
  }
}

void read_mail(char *player)
{
  GString *spool=g_string_new(g_Config.datapath->str);
  int fd;
  char buf[1024];
  FILE *f;
  g_string_append(spool,MAKROKOSMOS_MAIL_PATH);
  g_string_append(spool,player);
  fd=open(spool->str,O_RDONLY);
  if (fd>=0) {
    if (flock(fd,LOCK_EX)==0) {
      f=fdopen(fd,"r");
      if (f) {
        while (fgets(&buf[0],1024,f)) {
          buf[1023]='\0';
          buf[greater(strlen(&buf[0])-1,0)]='\0';
          ui_message(&buf[0]);
        }
        fclose(f);
      }
      unlink(spool->str);
      flock(fd,LOCK_UN);
    } else {
      ui_config_error("Error %d (%s) locking mail spool",errno);
    }
    close(fd);
  } else {
    if (errno==ENOENT) {
      ui_message("No mail.");
    } else {
      ui_config_error("Error %d (%s) reading mail spool",errno);
    }
  }
  g_string_free(spool,TRUE);
}

void check_chat(void)
{
  char chatline[1024];
  int numread;
  char *newline;
  if (flock(g_Chat_fd,LOCK_SH)==0) {
    lseek(g_Chat_fd,0,SEEK_CUR);
    while ((numread=read(g_Chat_fd,&chatline,1024))) {
      chatline[1023]='\0';
      if ((newline=strstr(&chatline[0],"\n"))) {
        newline[0]='\0';
        lseek(g_Chat_fd,0-(numread-((newline-chatline)+1)),SEEK_CUR);
        ui_show_chatline(&chatline[0]);
      } else {
        ui_error("Outsized line in chat.");
      }
    }
    flock(g_Chat_fd,LOCK_UN);
  }
}

void send_chat(void)
{
  int oldpos;
  GString *line=read_gstring(": ");
  if (line->len==0) {
    g_string_free(line,TRUE);
    return;
  }
  if (line->len+g_Player.name->len>=1021) {
    ui_message("Line too long for hyperspace.  Not sent.");
    g_string_free(line,TRUE);
    return;
  }
  if (flock(g_Chat_fd,LOCK_EX)==0) {
    oldpos=lseek(g_Chat_fd,0,SEEK_CUR);
    lseek(g_Chat_fd,0,SEEK_END);
    if (write(g_Chat_fd,g_Player.name->str,g_Player.name->len) !=
      g_Player.name->len) {
      ui_error("Error writing chatter string.");
    }
    if (write(g_Chat_fd,": ",2) != 2) {
      ui_error("Error writing chat string ': '");
    }
    if (write(g_Chat_fd,line->str,line->len) != line->len) {
      ui_error("Error writing chat string.");
    }
    if (write(g_Chat_fd,"\n",1) != 1) {
      ui_error("Error writing chat newline.");
    }
    if (oldpos!=-1) {
      lseek(g_Chat_fd,oldpos,SEEK_SET);
    } else {
      ui_error("Oldpos was -1.");
    }
    flock(g_Chat_fd,LOCK_UN);
  }
  g_string_free(line,TRUE);
}

void open_chat_file(void)
{
  GString *file=g_string_new(g_Config.datapath->str);
  g_string_append(file,MAKROKOSMOS_CHAT_PATH);
  g_string_append(file,MAKROKOSMOS_CHAT_FILE);
  g_Chat_fd=open(file->str,O_RDWR);
  if (g_Chat_fd==-1) {
    ui_error("Couldn't open chat file.");
  } else if (flock(g_Chat_fd,LOCK_SH)==0) {
    lseek(g_Chat_fd,0,SEEK_END);
    flock(g_Chat_fd,LOCK_UN);
  }
}
