/*
 *
 *  Makrokosmos sector file.  (C) 2000 Deekoo L., released under GPL.
 *
 *  Should contain all lowlevel routines pertaining to sectors,
 *  bodies, and zones.
 *
 */

#include <stdlib.h>
#include <math.h>
#include "microcosm.h"
#include "vectors.h"
#include "util.h"

int vector_distance_3d(struct coord_t c, struct coord_t d) 
{
  return sqrt(pow(c.x-d.x,2)+pow(c.y-d.y,2)+pow(c.z-d.z,2));
}

int numlen(int num)
{
  int x=0;
  if (num==0) return 1;
  while (num) {
    num /= 10;
    x++;
  }
  return x;
}

void string_to_coords(GString *s, struct coord_t *c)
{
  int x=0;
  *c=(struct coord_t){x:0,y:0,z:0};
  c->x = atoi(s->str);
  x+=numlen(c->x)+1;
  if (x<s->len) {
    c->y = atoi(&s->str[x]);
    x += numlen(c->y)+1;
    if (x<s->len) {
      c->z=atoi(&s->str[x]);
      /* N-dimensional code will eventually go here */
    }
  }
}

GString *coords_to_string(struct coord_t *c)
{
  GString *s=g_string_new("");
  g_string_sprintf(s,"%d/%d/%d",c->x,c->y,c->z);
  return s;
}

int vector_distance_s(GString *coords1, GString *coords2)
{
  struct coord_t c1,c2;
  string_to_coords(coords1,&c1);
  string_to_coords(coords2,&c2);
  return vector_distance_3d(c1,c2);
}

void determine_vector_pos(struct coord_t *src, struct coord_t *dest,
  struct coord_t *new, int length, int travelled)
{
  float	ratio=(float)travelled/(float)length;
  *new=*src;
  new->x+=(dest->x-src->x)*ratio;
  new->y+=(dest->y-src->y)*ratio;
  new->z+=(dest->z-src->z)*ratio;
}

int coords_oob(struct coord_t *c)
{
  return (c->x<0) || (c->y<0) || (c->z<0) ||
    (c->x>g_Config_Horizon) || (c->y>g_Config_Horizon) ||
    (c->z>g_Config_Horizon);
}

void randomize_coords(struct coord_t *c)
{
  c->x=randval_i(g_Config_Horizon);
  c->y=randval_i(g_Config_Horizon);
  c->z=randval_i(g_Config_Horizon);
}
